#include <SPI.h>
#include "src/Adafruit_GFX/Adafruit_GFX.h"
#include "src/Adafruit_SSD1306/Adafruit_SSD1306.h"
#include "src/IQS266/IQS266.h"

/*** Defines ***/
// AZP839A02 Requirements
#define TP_RDY 2
#define TP_ADR 0x44
#define LED1  A0
#define LED2  A1
#define LED3  A2
#define LED4  A3
// OLED Requirements
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13
// Coordinates
#define X_MIDDLE  63
#define Y_MIDDLE  31

/*** Types ***/
typedef enum {
  left,
  right,
  up,
  down
} Direction_t;

typedef union
{
  uint8_t modeByte;
  struct{
    uint8_t active:1;
    uint8_t lp:1;
    uint8_t draw:1;
    uint8_t proxTouch:1;
    uint8_t reservedForPrevious:4;   
  }current;
  struct{
    uint8_t reservedForCurrent:4;
    uint8_t active:1;
    uint8_t lp:1;
    uint8_t draw:1;
    uint8_t proxTouch:1;
  }previous;
} Mode_t;
/*** Macros ***/
#define mode (*(Mode_t *) &GPIOR0)

/*** Instances ***/
IQS266 tp; // Creat an IQS266 instance for the trackpad.
Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

/*** Variables ***/
unsigned int proxCounts = 0;
unsigned int proxDelta = 0;
unsigned int prevProxDelta = 0;
unsigned int xCoordinate_tp = 0;
unsigned int yCoordinate_tp = 0;
unsigned int xCoordinate_ol = 0;
unsigned int yCoordinate_ol = 0;
unsigned int prevXCoordinate_ol = 0;
unsigned int prevYCoordinate_ol = 0;
unsigned long lastActivityTime = 0;
unsigned long noActivityTimer = 0;
unsigned long firstModeTime = 0;
unsigned long modeTimer = 0;
bool displayState = true;

void setup() {
    // Start Serial communication.
  Serial.begin(9600);

  // Initialize the trackpad.
  while(!tp.begin(TP_ADR, TP_RDY));
  while(!tp.requestComms());
  // Acknowledge Reset.
  tp.acknowledgeReset(RESTART);
  // Set Zoom Timeout.
  // This is the amount of inactive time before entering low power.
  tp.setZoomTimeout(50, RESTART);
  // Set Prximity Settings.
  tp.setProxBaseValue(3, RESTART);
  tp.setProxAtiTarget(48, RESTART);
  tp.setProxSensitivity(2, RESTART);
  tp.disableProxDistribute(RESTART);
  // Set Touch Settings.
  tp.setTouchBaseValue(3, RESTART);
  tp.setTouchAtiTarget(35, RESTART);
  tp.setTouchSensitivity(27
  , RESTART);
  // Set Tap Settings.
  tp.setTapTimerLimit(50, RESTART);
  tp.setTapThreshold(35, RESTART);
  // Set Swipe Settings.
  tp.setSwipeTimerLimit(150, RESTART);
  tp.setSwipeThreshold(50, RESTART);
  // Autone after making settings.
  tp.autoTune(STOP);
  delay(100);
  
  // Do a startup routine.
  // Start OLED.
  display.begin(SSD1306_SWITCHCAPVCC);
  display.display();
 // delay(2000);
  display.clearDisplay();
  oledStartupRoutine();
  oledClear();
  // Display that initialization is complete and trackpad is ready.
  while(!tp.waitForReady());
  tp.setZoomTimeout(20, STOP);
  Serial.println("Trackpad Ready");
  display.clearDisplay();
  oledDisplayText("Ready", X_MIDDLE-25, Y_MIDDLE-5, 2, WHITE);
}

void loop() {
  // Read trackpad information.
  while(!tp.waitForReady());
  tp.updateSystemFlags(RESTART);
  tp.updateEvents(RESTART);
  tp.updateGestures(RESTART);
  tp.updateChannelStatus(RESTART);
  xCoordinate_tp = tp.readXCoordinate(RESTART);
  yCoordinate_tp = tp.readYCoordinate(RESTART);
  proxCounts = tp.readChannelCounts(CH_0, RESTART);
  prevProxDelta = proxDelta;
  proxDelta = tp.readChannelDelta(CH_0, STOP);

  // Map the trackpad coordinates to OLED coordinates.
  prevXCoordinate_ol = xCoordinate_ol;
  prevYCoordinate_ol = yCoordinate_ol;
  xCoordinate_ol = yCoordinate_tp/2;
  yCoordinate_ol = xCoordinate_tp/2;

  if (proxDelta<2)
    writeLeds(0);
  else if(proxDelta<=4)
    writeLeds(1);
  else if (proxDelta<=8)
    writeLeds(2);
  else if (proxDelta<=12)
    writeLeds(3);
  else if (proxDelta>12)
    writeLeds(4);


  if (tp.channels.touch.ch0)
  {
    // Reset activity tracking.
    lastActivityTime = millis();
    noActivityTimer = 0;
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.proxTouch = true;
    Serial.println(mode.modeByte);
    if(!mode.previous.proxTouch)
    {
      firstModeTime = millis();
      modeTimer = 1; 
      Serial.println("Prox Touch");
      display.invertDisplay(displayState);
      displayState = !displayState;
    }
    else
      modeTimer = millis() - firstModeTime;
 
    // Update OLED.
    if((modeTimer >= 1000) && (modeTimer%1000 < 50))
    {
      oledClear();
      oledDisplayText(String(modeTimer/1000), X_MIDDLE-5, Y_MIDDLE-5, 2, WHITE);
      display.display();
    }
  }
  else if(tp.events.swipe)
  {
    // Reset activity tracking.
    lastActivityTime = millis();
    noActivityTimer = 0;
    modeTimer = 0;
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.active = true;
    // Distinguish which swip it was.
    if(tp.gestures.swipeUp)
    {
      // Trackpad swipeUp is oledSwipeRight.
      Serial.println("Swipe Right");
      oledSwipeLine(yCoordinate_ol, right, 254);
    }
    else if (tp.gestures.swipeDown)
    {
      // Trackpad swipeDown is oledSwipeLeft.
      Serial.println("Swipe Left");
      oledSwipeLine(yCoordinate_ol, left, 254);
    }
    else if (tp.gestures.swipeLeft)
    {
      // Trackpad swipeLeft is oledSwipeUp.
      Serial.println("Swipe Up");
      oledSwipeLine(xCoordinate_ol, up, 254);
    }
    else if (tp.gestures.swipeRight)
    {
      // Trackpad swipeRight is oledSwipeDown.
      Serial.println("Swipe Down");
      oledSwipeLine(xCoordinate_ol, down, 254);
    }
  }
  else if (tp.events.tap)
  {
    // Reset activity tracking.
    lastActivityTime = millis();
    noActivityTimer = 0;
    modeTimer = 0;
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.active = true;
    // Display Tap on OLED.
    Serial.println("Tap");
    oledTap(xCoordinate_ol, yCoordinate_ol);
  }
  else if (tp.events.lowPower && tp.systemFlags.lpActive)
  {
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.lp = true;
    // Update display.
    Serial.println("Low Power Mode");
    display.clearDisplay();
    oledDisplayText("Low Power", X_MIDDLE-50, Y_MIDDLE-5, 2, WHITE);
  }
  else if (mode.current.lp && tp.events.prox)
  {
    // Reset activity tracking
    lastActivityTime = millis();
    noActivityTimer = 0;
    modeTimer = 0;
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.active = true;
    // Update OLED.
    Serial.println("Active Mode");
    Serial.println(mode.modeByte);
    display.clearDisplay();
    oledDisplayText("Active", X_MIDDLE-30, Y_MIDDLE-5, 2, WHITE);
    /* Note on Low Power
     * The IQS266 sets the LP EVENT event flag and the LP ACTIVE system flag 
     * upon enter low power. The LP EVENT event flag is cleared after reading
     * the events flag. The LP EVENT event flag is not set upon exiting low power mode.
     * Low power is exiting upon detecting a prox, so if the PROX EVENT flag is set
     * then the IQS266 must be out of low power.
     */
  }
  else if (tp.channels.flagByte & 0x7E)
  {
    // Reset activity tracking
    lastActivityTime = millis();
    noActivityTimer = 0;
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.draw = true;
    if (!mode.previous.draw)
    {
      firstModeTime = millis();
      modeTimer = 1;
      oledClear();
      Serial.println("Draw");
    }
    else
      modeTimer = millis() - firstModeTime;

    if (modeTimer > 400)
    {
        display.drawLine(prevXCoordinate_ol, prevYCoordinate_ol, xCoordinate_ol, yCoordinate_ol, WHITE);
        display.display();
    }
  }
  else if (!(tp.channels.flagByte & 0x7E) && tp.events.touch)
  {
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.active = true;
    // Clear mode timer and display.
    modeTimer = 0;
    oledClear();
  }
  else if (!mode.current.lp && (noActivityTimer > 3000))
  {
    display.clearDisplay();
    oledDisplayText("Ready", X_MIDDLE-25, Y_MIDDLE-5, 2, WHITE);
  }
  noActivityTimer = millis() - lastActivityTime;

  //LP Animation.
  if(mode.current.lp)
  {
    if(!mode.previous.lp)
    {
      Serial.println(mode.modeByte);
      firstModeTime = millis();
      modeTimer = 1;
    }
    else
      modeTimer = millis() - firstModeTime;

    if(modeTimer > 300)
    {
      if((modeTimer%1000) < 300)
      {
        oledDisplayText("z", X_MIDDLE+53, Y_MIDDLE-26, 1, BLACK);
        oledDisplayText("z", X_MIDDLE+53, Y_MIDDLE-10, 1, WHITE);
      }
      else if((modeTimer%1000) < 600)
      {
        oledDisplayText("z", X_MIDDLE+53, Y_MIDDLE-10, 1, BLACK);
        oledDisplayText("z", X_MIDDLE+47, Y_MIDDLE-18, 1, WHITE);
      }
      else if((modeTimer%1000) < 900)
      {
        oledDisplayText("z", X_MIDDLE+47, Y_MIDDLE-18, 1, BLACK);
        oledDisplayText("z", X_MIDDLE+53, Y_MIDDLE-26, 1, WHITE);
      }
    }
    // Update mode flags.
    mode.modeByte = (mode.modeByte<<4);
    mode.current.lp = true;
  }
}

/*** Functions ***/
/* @brief Writes the LEDs according to a specified level between 0 and 4, both inclusive.*/
void writeLeds(unsigned char level)
{
      digitalWrite(LED4, (level & 0x07));
      digitalWrite(LED3, (level & 0x06));
      digitalWrite(LED2, (level & 0x06) && (level & 0x05));
      digitalWrite(LED1, (level & 0x04));
}

/* @brief Cycles the leds back and forth. */
void cycleLeds(void)
{
  writeLeds(1);
  delay(200);
   writeLeds(2);
  delay(200);
   writeLeds(3);
  delay(200);
   writeLeds(4);
  delay(200);
   writeLeds(3);
  delay(200);
   writeLeds(2);
  delay(200);
   writeLeds(1);
  delay(200);
   writeLeds(0);
}

/* @brief Clears the OLED display. */
void oledClear()
{
  display.clearDisplay();
  display.display();
}

void oledStartupRoutine(void)
{
  // Flash IQS266.
  //oledDisplayText("IQS266", X_MIDDLE-40, Y_MIDDLE-5, 2, WHITE);
  cycleLeds();
 // delay(1000);
  panelFlash(3);
 // delay(1000);
  display.clearDisplay();
  display.display();
  // Tap Demonstration.
  oledDisplayText("TAP", X_MIDDLE-20, Y_MIDDLE-5, 2, WHITE);
 // delay(500);
  for(int i=0; i<4; i++)
  {
    unsigned char xRand = random(5, 123);
    unsigned char yRand = random(5, 59);
    oledTap(xRand, yRand);
 //   delay(200);
    oledDisplayText("TAP", X_MIDDLE-20, Y_MIDDLE-5, 2, WHITE);
    display.display();
  }
  display.clearDisplay();
  display.display();
 // delay(500);
  // Swipe demonstration.
  oledDisplayText("SWIPE", X_MIDDLE-30, Y_MIDDLE-5, 2, WHITE);
 // delay(500);
  oledSwipeLine(5, left, 254);
 // delay(200);
  oledSwipeLine(58, right, 254);
 // delay(200);
  oledSwipeLine(5, down, 254);
//  delay(200);
  oledSwipeLine(120, up, 254);
 // delay(200);
  display.clearDisplay();
  display.display();
//  delay(500);
  // Draw demonstartion.
  oledDisplayText("DRAW", X_MIDDLE-25, Y_MIDDLE-5, 2, WHITE);
 // delay(500);
  int xCoord = random(0, 127);
  char yCoord = random(0, 63);
    int prevX = xCoord;
  char prevY = yCoord;
  char xInc = random(-5, 5);
  char yInc = random(-5, 5);
  for(int i=0; i<96; i++)
  {
    display.drawLine(prevX, prevY, xCoord, yCoord, WHITE);
    display.display();
    prevX = xCoord;
    prevY = yCoord;
    if(i%5 == 0)
    {
      xInc = random(5) * (xInc/abs(xInc));
      yInc = random(5) * (yInc/abs(yInc));
    }
    xCoord = xCoord + xInc;
    yCoord = yCoord + yInc;
    if(xCoord <=0)
    { 
      xCoord = 0;
      xInc = random(0, 5);
    }
    else if (xCoord >= 127)
    {
      xCoord = 127;
      xInc = random(-5, 0);
    }
    if(yCoord <=0)
    {
      yCoord = 0;
      yInc = random(0, 5);
    }
    else if (yCoord >= 63)
    {
      yCoord = 63;
      yInc = random(-5, 0);
    }
  //  delay(30);
  }
  display.clearDisplay();
  display.display();
//  delay(500);
}


void oledFlash(unsigned char iterations)
{
  iterations *= 2;
  bool displayState = true;
  for (int i=0; i<iterations; i++)
  {
    display.invertDisplay(displayState);
    displayState = !displayState;
    delay(100);
  }
}

void panelFlash(unsigned char iterations)
{
  iterations *= 2;
  bool displayState = true;
  for (int i=0; i<iterations; i++)
  {
    display.invertDisplay(displayState);
    displayState ? writeLeds(4): writeLeds(0);
    displayState = !displayState;
    delay(100);
  }
}

void oledDisplayText(String text, unsigned char xCoord, unsigned char yCoord)
{
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(xCoord, yCoord);
  display.println(text);
  display.display();
}

void oledDisplayText(String text, unsigned char xCoord, unsigned char yCoord, unsigned char textSize, unsigned char color)
{
  display.setTextSize(textSize);
  display.setTextColor(color);
  display.setCursor(xCoord, yCoord);
  display.println(text);
  display.display();
}


void oledTap(uint8_t xCoord, uint8_t yCoord)
{
  for(int i=1; i<6; i++)
  {
    display.drawCircle(xCoord, yCoord, (2*i), WHITE);
    display.display();
    delay(5);
    display.drawCircle(xCoord, yCoord, (2*(i)), BLACK);
  }
  display.display();
}

void oledSwipeCircle(unsigned char coord, Direction_t dir, uint8_t swipeSpeed)
{
  switch(dir){
    case left:
      for(int i=7; i>=0; i--)
      {
        display.fillCircle((i*16)+8, coord, 1, WHITE);
        display.drawLine((i*16)+8, coord, ((i*16)+16), coord, WHITE);
        display.display();
        delay(255 - swipeSpeed);
        display.fillCircle((i*16)+8, coord, 1, BLACK);
        display.drawLine((i*16)+8, coord, ((i*16)+16), coord, BLACK); 
      }
      display.display();
      break;
    case right:
      for(int i=0; i<8; i++)
      {
        display.fillCircle((i*16)+8, coord, 1, WHITE);
        display.drawLine((i*16)+8, coord, ((i*16)), coord, WHITE);
        display.display();
        delay(255 - swipeSpeed);
        display.fillCircle((i*16)+8, coord, 1, BLACK);
        display.drawLine((i*16)+8, coord, ((i*16)), coord, BLACK); 
      }
      display.display();
      break;
     case up:
      for(int i=7; i>=0; i--)
      {
        display.fillCircle(coord, (i*8)+4, 1, WHITE);
        display.drawLine(coord, (i*8)+4, coord, ((i*8)+12), WHITE);
        display.display();
        delay(255 - swipeSpeed);
        display.fillCircle(coord, (i*8)+4, 1, BLACK);
        display.drawLine(coord, (i*8)+4, coord, ((i*8)+12), BLACK); 
      }
      display.display();
      break;
     case down:
      for(int i=0; i<8; i++)
      {
        display.fillCircle(coord, (i*8)+4, 1, WHITE);
        display.drawLine(coord, (i*8)+4, coord, ((i*8)-4), WHITE);
        display.display();
        delay(255 - swipeSpeed);
        display.fillCircle(coord, (i*8)+4, 1, BLACK);
        display.drawLine(coord, (i*8)+4, coord, ((i*8)-4), BLACK); 
      }
      display.display();
      break;
  }
}

void oledSwipeLine(unsigned char coord, Direction_t dir, uint8_t swipeSpeed)
{
  switch(dir){
    case left:
      for(int i=7; i>=0; i--)
      {
        display.drawLine((i*16), coord, ((i*16)+16), coord, WHITE);
        display.display();
        delay(255-swipeSpeed);
      }
      display.drawLine(0, coord, 127, coord, BLACK);
      display.display();
      break;
    case right:
      for(int i=0; i<8; i++)
      {
        display.drawLine((i*16)+16, coord, ((i*16)-16), coord, WHITE);
        display.display();
        delay(255-swipeSpeed);
      }
      display.drawLine(0, coord, 127, coord, BLACK); 
      display.display();
      break;
     case up:
      for(int i=7; i>=0; i--)
      {
        display.drawLine(coord, (i*8), coord, ((i*16)+8), WHITE);
        display.display();
        delay(255-swipeSpeed);
      }
      display.drawLine(coord, 0, coord, 63, BLACK); 
      display.display();
      break;
     case down:
      for(int i=0; i<8; i++)
      {
        display.drawLine(coord,(i*8), coord, ((i*16)+8), WHITE);
        display.display();
        delay(255-swipeSpeed);
      }
      display.drawLine(coord, 0, coord, 63, BLACK); 
      display.display();
      break;
  }
}
