# IQS266 Library
The IQS266 library is a library which allows easy setup and interaction with the Azoteq IQS266, 2x3 Channel projected
cpacitive trackpad controller. The IQS266 is ideal for small trackpads which do not require high resolution as it provides easy on chip
gesture detection as well as coordinate calculation. 

The IQS266 requires very little initialization as its default startup settings already provide excellent trackpad control.
Furthermore, the easy to use and understand library methods allow the user to quickly customize the trackpad for the intended
purpose. See the list of functions below.

Since the IQS266 interacts with the MCU via I2C communication, this library is based on the standard Arduino
"Wire.h" library and is thus supported by any hardware which is supported by the "Wire.h" library. The "Wire.h" library
is included within the "IQS266.h" library and does not need to be included again. Furthermore, a Wire instance is 
created in the IQS266::begin() function and does not need to be created in the Setup() section of the Arduino sketch. 
The user does not need to interact with the "Wire.h" library at all.

# IQS266 Device
Information on the IQS266 device can be found her: http://www.azoteq.com/2017-02-09-07-38-01/proxsense/11-products/385-iqs266.html
The datasheet can be downloaded from this link: http://www.azoteq.com/images/stories/pdf/iqs266_datasheet.pdf
Any application notes can be found here: http://www.azoteq.com/design/applications.html 

# Methods:
Method names are mostly self explanatory. However, there is also a short commented header above each method
in the "IQS266.cpp" file which offers a brief explanation and description of parameters and return values. Also
see the "IQS266.h" file for all the method prototypes.

begin
requestComms
waitForReady
checkReset
acknowledgeReset
autoTune
setStreamMode
setEventMode
readSystemFlags
updateSystemFlags
readEvents
updateEvents
readGestures
updateGestures
readTrackpadFlags
updateTrackPadFlags
readProxByte
readTouchByte
readChannelStatus
updateChannelStatus
readXCoordinate
readYCoordinate
setTouchSensitivity
setTouchThresholds
setProxSensitivity
setProxThreshold
enableProxEvent
enableTouchEvent
enableTrackPadEvent
enableTapEvent
enableSwipeEvent
enableAllEvents
disableProxEvent
disableTouchEvent
disableTrackPadEvent
disableTapEvent
disableSwipeEvent
disableAllEvents
enableChannel
disableChannel
readChannelCounts
readChannelLta
readChannelDelta
setTapTimerLimit
setTapThreshold
setSwipeTimerLimit
setSwipeThreshold
enableProxDistribute
disableProxDistribute
setEventMask
setProxAtiTarget
setTouchAtiTarget
setProxBaseValue
setTouchBaseValue
setReportRateNM
setReportRateLP
setNPSegmentRate
setZoomTimeout
disableHaltTimeout
setHaltTimeout

# I2C Communication:
The IQS266 communicates with the master MCU through I2C communication. However, the IQS266 has an additional "Ready" line which
can let the master know when it has information available. This is particularly useful as it allows the MCU to service the IQS266
on an interrupt basis rather than polling it. More on "Event Mode" and "Stream Mode" later.
NOTE: DO NOT PERFORM I2C COMMUNICATION IN AN INTERRUPT SERVICE ROUTINE, THIS CAUSES THE MCU TO CRASH. I don't know why.

Most functions of th IQS266 library have a boolean parameter as the last parameter named stopOrRestart. This boolean indicates
wether the current communication window must be kept open or closed. Use STOP or RESTART as defined in the library header.
When calling multiple functions in a block of code with the same IQS266 instance all functions except for the last should 
use RESTART and the last function should then use STOP. See the accompanied examples.

# Default Settings:
The default startup settings of th IQS266 work well. However, some settings such as touch sensitivity depend largely on the
trackpad design and the overlay on the touch interface. These settings can be adjusted at initilialzation after the begin
method has been called.

# Event Mode and Stream Mode:
The IQS266 starts up in Stream Mode, however, Event Mode is very convenient as the user can select which events are important.
Event mode means that the device only requests communication when it has detected an event as allowed by the enabled events.
Communication is requested by pulling the ready pin LOW, the ready pin will continue to toggle on and off until communication is established.
Event mode is very convenient as it allows the MCU to carry on with its own business until the IQS266 detects an event and pulls the ready pin LOW.
The ready pin on the Arduino can then trigger an external interrupt or a pin change interrupt.
NOTE: DO NOT PERFORM I2C COMMUNICATION IN AN INTERRUPT SERVICE ROUTINE, THIS CAUSES THE MCU TO CRASH. I don't know why.

In Stream Mode the IQS device pulls the ready line low in specified intervals which lets the MCU know that the latest sensing
information is now avialable to be read. This is done regardless of whether an event is detected or not. In this way the MCU can
decide when it wants to read information polling the ready line and reading the information it needs as soon as it is avilable. Unless
the MCU checks on the IQS device regularly it can miss events.

# Requesting Communication:
In stream mode the ready pin is pulled low frequently, when the ready line is LOW the MCU can innitiate communication with the IQS device.
The waitForReady() function can be used to wait for the ready pin to go LOW, the waitForReady() function waits until the ready pin is
LOW and then returns a TRUE. If the ready pin does not go LOW within 100ms it returns FALSE. The function can be put in a while loop to
continue calling it until the ready pin is pulled LOW.

In event mode there might be long periods where no event occurs, then the ready pin is not pulled low by the IQS device. The MCU can request
communication by briefly pulling the ready pin low and then waiting for the IQS device to pull the ready pin low. This is done by the
requestComms() function. A TRUE is returned if the IQS responds within 100ms, otherwise a FALSE is returned. Use the function in a while loop
to call it until communication is established. After the requestComms() function returns a TRUE, the MCU can proceed with other functions.
In event mode any block of IQS266 library functions should be preceeded by the requestComms() function unless it is certain that the IQS device
has solicited the communication. The last function in any block of functions should then use STOP as its final argument.

# Events
In Event Mode an event causes the IQS266 to pull the Ready pin LOW if that event is enabled. This event is then stored in the Events register
until the current communication window is closed.
In Stream Mode any occuring event is kept in the Events register until the next conversion cycle which then loads the new events.
The events Register can be read with the readEvents() function. This returns a byte. The definitions PROX_EVENT_BIT, TOUCH_EVENT_BIT, TP_EVENT_BIT,
ATI_EVENT_BIT, TAP_EVENT_BIT and SWIPE_EVENT_BIT, as defined in "IQS266.h", can be used to determine which event bits are set and thus
which events have ocurred.

To really understand how the events work the IQS266 datasheet can be consulted. Here is a brief overview of how events are generated.
Prox -> 	Single event when the PROX_THRESHOLD is entered and a single event when the PROX_THRSHOLD is exited. Therefore, when
			bringing a hand close to a capacitive electrode and then moving it away, two events will be generated, once for entering
			the PROX_THRESHOLD and once for exiting the PROX_THRESHOLD. PROX_THRESHOLD can be set with the setProxSensitivity function,
			the default is 10.
Touch ->	Single event when the TOUCH_THRESHOLD is entered and single event when the TOUCH_THRESHOLD is exited. Therefore, when touching 
			the capacitive sensing area and then releasing it, two events will be generated, one for touching and one for releasing.
			TOUCH_THRESHOLD can be set with the setTouchSensitivity() functions, default is 15 for CH_1, CH_2 and CH_3.
TrackPad ->	When the TP Event is enabled the IQS266 device enters Stream Mode as long as a touch is present on any channel. Thus,
(TP)		data can be read continuously from the IQS266 device as long as a touch is present. With an interrupt
			based application, no provision needs to be made for this, the interrupt handles the streaming easily. See the example programs.
ATI   ->	Single event at start of an ATI procedure and single event at completion of an ATI procedure.
			This is for more advanced users, see the datasheet for information.
Tap	  ->	A single Tap event is generated when a Touch event lasts for less time than the Tap Timer Limit value, the default value is 150ms.
			And the tap takes up less space than a square with a coordinate length of the Tap Threshold, the default is 30. The Tap Timer Limit and the 
			Tap Threshold can be set with the setTapTimerLimit() and setTapThreshold() functions.
Swipe ->	A single Swipe event is generated when a Touch moves along the trackpad for and amount of coordinates larger than the Swipe Threshold value 
			and the touch is present for a time less than the Swipe Timer Limit value. The Swipe Threshold value can be set with the setSwipeThreshold()
			function, the default value is 48. The Swipe Timer Limit value can be set with the setSwipeTimerLimit() function, the default value is 300ms.
			The IQS266 device generates a swipe event if a swipe occurs in the positive or negative diresctions of the X and Y axes, i.e. X+, X-, Y+, Y-.
			Read the TP_FLAGS register and use the SWIP_UP_BIT, SWIPE_DOWN_BIT, SWIPE_LEFT_BIT and SWIPE_RIGHT_BIT definitions to determine in which
			direction the swipe occurred.
			
Events Example: For this example let the Prox, Touch and Tap events be enabled. A user birngs his hand from outside the PROX_THRESHOLD and 
				touches the capacitive sensing area for 3ms, releases and then moves his hand out of the PROX_THRESHOLD.
				A total of 5 events will be generated in the following order: 
				1. Breach Prox Threshold
				2. Breach Touch Threshold
				3. Release Touch Threshold
				4. Tap
				5. Release Prox Threshold.
				Given the above situation, the readEvents() method can be used with the TAP_EVENT_BIT to determine if a tap has occurred.
				
That covers the basics. When in doubt, consult the device datasheet. Azoteq also offers an evaluation kit, configuration tool and free GUI based PC software
which makes it easy to configure settings from basic to advanced and visualize what is happening inside the device.
See this page for the EV kit and PC software: hhttp://www.azoteq.com/design/evaluation-kits.html
