/*  IQS266: 6 Channel capacitive touch track-pad example.
 *  This example uses the Azoteq IQS266 capacitive touch controller with a 6 channel trackpad to stream coordinates.
 *  The example starts the IQS266 with its default settings, then sets the event mask and puts the IQS266 into event mode.
 *  A pin change interrupt is enabled on the ready pin with the EnableInterrupts library.
 *  
 *  Hardware:
 *  - IQS266 with 2x3 track-pad
 *  - Arduino pin 8 - Ready
 *  - Arduino pin A4 - SDA
 *  - Arduino pin A5 - SCL
 */

/***** Library Includes *****/
#include <IQS266.h>
#include <EnableInterrupt.h>


/***** Object Declarations *****/
IQS266 trackPad;

/***** Constant Definitions *****/
#define TRACKPAD_RDY 8
#define TRACKPAD_ADDR 0x44

/***** Global Variables *****/
unsigned char xCoordinate = 0;
unsigned char yCoordinate = 0;
bool trackPadInterrupt = false;

/***** Function Prototypes *****/
void trackPadInterruptHandler(void);

/***** Setup *****/
void setup() {
  // Set ready pin as INPUT.
  pinMode(TRACKPAD_RDY,INPUT);

  // Start serial communication for output.
  Serial.begin(9600);

  // Initialize the track-pad.
  while(!trackPad.begin(TRACKPAD_ADDR, TRACKPAD_RDY));      // Constructor assigns ready pin an address to IQS266 object, returns 1 if successful.
  while(!trackPad.requestComms());                          // Request communication for additional settings.
  trackPad.disableHaltTimeout(RESTART);                     // Disable halt timeout
  trackPad.disableAllEvents(RESTART);                       // All events are enabled by defau;ts, we only want the TrackPad Event.
  trackPad.enableTrackPadEvent(RESTART);                    // Enable the TrackPad event, this streams information whenever there is a touch.
  trackPad.setEventMode(STOP);                              // Set the IQS266 into event mode.

  // Enable a pin change interrupts on the ready pin
  enableInterrupt(TRACKPAD_RDY, trackPadInterruptHandler, FALLING);
  
  Serial.println("Track-Pad Ready");         // Let the user know initialization is complete.
}

/***** Main Loop *****/
void loop() {
  if(trackPadInterrupt)
  {
    // Ready should be low so jump right into communication.
    xCoordinate = trackPad.readXCoordinate(RESTART);
    yCoordinate = trackPad.readYCoordinate(STOP);
    // Print the coordinates to the terminal.
    Serial.print(xCoordinate);      // Print the x-coordinate.
    Serial.print(" ; ");
    Serial.println(yCoordinate);    // Print the y-coordinate
    delay(100);
  
    trackPadInterrupt = false;
  }
}

void trackPadInterruptHandler(void)
{
  trackPadInterrupt = true;
}
